/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fr.emse.gitlab.saref.checkers;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.nio.charset.MalformedInputException;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.PathMatcher;
import java.util.stream.Collectors;

import org.apache.jena.atlas.RuntimeIOException;
import org.apache.jena.riot.Lang;

import fr.emse.gitlab.saref.SAREF;
import fr.emse.gitlab.saref.SAREFPipelineException;
import fr.emse.gitlab.saref.entities.SAREFExample;
import fr.emse.gitlab.saref.managers.RepositoryManager;

/**
 * Checks TS 103 673 Clause 9.6.1: The examples directory
 * 
 */
public class Clause_9_6_1_Checker extends AbstractClauseChecker {

	private static final PathMatcher TTL_MATCHER = FileSystems.getDefault().getPathMatcher("glob:**/*.ttl");

	private static enum MESSAGE {
		directories, one, ttl, turtle, write_error;
	}

	public Clause_9_6_1_Checker(RepositoryManager repositoryManager) {
		super(repositoryManager, Clause_9_6_1_Checker.class);
	}

	@Override
	public void checkClause() throws SAREFPipelineException {
		File dir = new File(repository.getDirectory(), "examples");
		if (!dir.isDirectory()) {
			return;
		}
		if (dir.list().length < 1) {
			log(getMessage(MESSAGE.one));
		}

		try {
			String directories = Files.walk(dir.toPath()).filter(p -> {
				try {
					return p.toFile().isDirectory() && !Files.isSameFile(dir.toPath(), p);
				} catch(IOException ex) {
					return false;
				}
			}).map(p -> p.toString()).collect(Collectors.joining(", "));
			if(directories.length()>0) {
				logError(getMessage(MESSAGE.directories, directories));
			}

			String nonTtl = Files.walk(dir.toPath(), 1).filter(p -> {
				try {
					return p.toFile().isFile() && !TTL_MATCHER.matches(p);
				} catch (Exception ex) {
					return false;
				}
			}).map(p -> p.getFileName().toString()).collect(Collectors.joining(", "));
			if(nonTtl.length()>0) {
				logError(getMessage(MESSAGE.ttl, nonTtl));
			}

			Files.walk(dir.toPath(), 1).filter(p -> {
				return TTL_MATCHER.matches(p);
			}).forEach(p -> {
				SAREFExample example = new SAREFExample(version, p);
				try (FileInputStream input = new FileInputStream(p.toFile())) {
					example.getModel().read(input, SAREF.BASE, Lang.TTL.getLabel());
				} catch (Exception ex) {
					if (ex instanceof RuntimeIOException && ex.getCause() instanceof MalformedInputException) {
						try (InputStreamReader input2 = new InputStreamReader(new FileInputStream(p.toFile()),
								Charset.defaultCharset())) {
							example.getModel().read(input2, SAREF.BASE, Lang.TTL.getLabel());
						} catch (Exception ex2) {
							String msg = getMessage(MESSAGE.turtle);
							logError(msg, ex2);
							return;
						}
					} else {
						String msg = getMessage(MESSAGE.turtle);
						logError(msg, ex);
						return;
					}
				}
				version.getExamples().put(example.getName(), example);
			});
		} catch (IOException ex) {
			throw new SAREFPipelineException(ex);
		}
	}

}
